@@ -21,6 +21,8 @@ std::string sd_load_merges();
 std::string sd_load_t5();
 std::string sd_load_umt5();
 std::string sd_load_qwen2_merges();
+std::string sd_load_mistral_merges();
+std::string sd_load_mistral_vocab_json();
 
 #include "flux.hpp"
 #include "stable-diffusion.cpp"
@@ -159,6 +161,26 @@ std::string sd_load_qwen2_merges()
     qwenmergesstr = read_str_from_disk(filepath);
     return qwenmergesstr;
 }
+std::string sd_load_mistral_merges()
+{
+    static std::string mistralmergesstr;  // cached string
+    if (!mistralmergesstr.empty()) {
+        return mistralmergesstr;  // already loaded
+    }
+    std::string filepath = executable_path + "embd_res/mistral2_merges_utf8_c_str.embd";
+    mistralmergesstr = read_str_from_disk(filepath);
+    return mistralmergesstr;
+}
+std::string sd_load_mistral_vocab_json()
+{
+    static std::string mistralvocabstr;  // cached string
+    if (!mistralvocabstr.empty()) {
+        return mistralvocabstr;  // already loaded
+    }
+    std::string filepath = executable_path + "embd_res/mistral2_vocab_json.embd";
+    mistralvocabstr = read_str_from_disk(filepath);
+    return mistralvocabstr;
+}
 std::string sd_load_t5()
 {
     static std::string t5str = "";
@@ -916,7 +938,7 @@ sd_generation_outputs sdtype_generate(const sd_generation_inputs inputs)
 
         if(!sd_is_quiet && sddebugmode==1)
         {
-            printf("\nImageGen References: RefImg=%d Wan=%d Photomaker=%d\n",reference_imgs.size(),wan_imgs.size(),photomaker_imgs.size());
+            printf("\nImageGen References: RefImg=%zu Wan=%zu Photomaker=%zu\n",reference_imgs.size(),wan_imgs.size(),photomaker_imgs.size());
         }
     }
 
