@@ -138,11 +138,6 @@ struct ggml_tensor * build_attn_mask(ggml_context * ctx, orpheus_context * octx,
 
 orpheus_context * build_new_orpheus_context(orpheus_model * model, int n_threads, bool use_cpu) {
     orpheus_context * octx = new orpheus_context(model, n_threads);
-    if (!use_cpu) {
-#ifdef GGML_USE_METAL
-        octx->backend = ggml_backend_metal_init();
-#endif
-    }
     octx->backend_cpu = ggml_backend_cpu_init();
     octx->set_threads();
     octx->build_schedule();
@@ -153,9 +148,6 @@ orpheus_context * build_new_orpheus_context(orpheus_model * model, int n_threads
 void orpheus_runner::orpheus_kv_cache_init() {
     ggml_backend_buffer_type_t buft = nullptr;
     if (octx->backend != nullptr) {
-#ifdef GGML_USE_METAL
-        buft = ggml_backend_metal_buffer_type();
-#endif
     } else {
         buft = ggml_backend_cpu_buffer_type();
     }
